%{
/**CFile***********************************************************************

  FileName    [lex.l]

  PackageName [ltl2smv]

  Synopsis    [The LTL to SMV Translator -- Lexical Analyzer]

  Description [This file provides routines for lexical analysis of LTL formulas.]

  SeeAlso     []

  Author      [Adapted to NuSMV by Marco Roveri]

  Copyright   [ Copyright (c) 1998 by ITC-IRST and Carnegie Mellon University.
  All Rights Reserved. This software is for educational purposes only.
  Permission is given to academic institutions to use, copy, and modify
  this software and its documentation provided that this introductory
  message is not removed, that this software and its documentation is
  used for the institutions' internal research and educational purposes,
  and that no monies are exchanged. No guarantee is expressed or implied
  by the distribution of this code.
  Send bug-reports and/or questions to: nusmv@irst.itc.it. ]

******************************************************************************/

#include <stdio.h>
#include "l2s.h"
#include "parser.h"

extern void inc_linenumber();
%}

W    [ \t]
L    [\n]
  
%%
{W}+          ;
{L}           {inc_linenumber();}
"|"           return OR;
"&"           return AND;
"->"          return IMPLY;
"<->"         return EQUIV;
"!"           return NEG; 
"X"           return NEXT;
"F"           return FUTURE;
"G"           return GLOBAL;
"U"           return UNTIL;
"+"           return PLUS;
"-"           return MINUS;
"*"           return TIMES;
"/"           return DIVIDE;
"mod"         return MOD;
"="           return EQUAL;
"<="          return LE;
">="          return GE;
"<"           return LT;
">"           return GT;
"union"       return UNION;
"in"          return SETIN;
[A-Za-z_][A-Za-z0-9_.\$#-]*  return ATOM;
[0-9]+        return NUMBER;
.             return yytext[0];
